package agents;
 
import sim.engine.Steppable;
import sim.engine.SimState;
import sim.util.Bag;
import sim.util.Int2D;
 
public class Agent implements Steppable{
        public int x;     // x coordinate of agent
        public int y;     // y coordinate of agent
        public int xdir;  // the number of spaces moved along the x-axis
                          // on each time step
        public int ydir;  // the number of spaces moved along the y-axis
                                         // on each time step 
        public boolean randomize = false; // if an agent collides with another

                                    //randomize the directions.
        
        /*
         * The data and methods below implement probabilistic movement where we
         * can control the direction of movement.
         */
        
        public int orientation;
        public double[][] movementProbs = null;
        public int[][] orientationLookUp = null;
        public Int2D[][] directionLookUp;
        public Int2D[] randomMovement;
        
        public int lookUpOrientation (int xdir, int ydir){
            return orientationLookUp[xdir+1][ydir+1];
        }

        public Int2D changeDirection(int orientation, int direction) {
            return directionLookUp[orientation][direction];
        }

        public int findDirction(SimState state){
            double x = state.random.nextDouble();
                  if(x < movementProbs[0][1]) //north
                      return 0;
                  else if(x < movementProbs[0][2]) //northeast                      
                	  return 1;
                  else if(x < movementProbs[1][2]) //east
                       return 2;
                  else if(x < movementProbs[2][2]) //southeast
                       return 3;
                  else if(x < movementProbs[2][1]) //south
                       return 4;
                  else if(x < movementProbs[2][0]) //southwest
                       return 5;
                  else if(x < movementProbs[1][0]) //west
                       return 6;
                  else
                       return 7; //northwest by default
        }

        public Int2D newDirection(SimState state){
                  Int2D dir;
                  int direction = findDirction(state);
                  dir = changeDirection(orientation, direction);
                  orientation =lookUpOrientation (dir.x,dir.y);
                  return dir;
        }
        
        /*
         * End of new data and methods for probabilistic movement.
         */



        /**
         * Agent constructor
         */
        public Agent(int x, int y, int xdir, int ydir){
                this.x = x;
                this.y = y;
                this.xdir = xdir;
                this.ydir = ydir;
        }

      /**
       *  Only movement is described here. Mating behavior is in the subclass, Person. 
       *  This movement code was adapted from some other code with a lot more options for movement, so it is a little wonky.
       */
      public void step(SimState state){
             AgentsSimulation as = (AgentsSimulation)state; 
             if(as.useSpace){//only do this if space is important. 
  
            	 Bag bag = as.agentsSpace.getObjectsAtLocation(x, y); 
                                       // get all the agents at x, y
            	 if (bag != null && bag.numObjs > 1)  // if there is more than 1, make them randomize 
            		 for(int i=0;i<bag.numObjs;i++) {// For every agent in the bag
            			 Agent a = (Agent)(bag.objs[i]); // Convert it to an agent
            			 a.randomize = true; // Tell it to randomize its direction
            		 }
             
             
             
             Bag neighbors = new Bag();    
             neighbors = as.agentsSpace.getNeighborsMaxDistance(x, y, as.searchRadius, false, neighbors, null, null);

             if (randomize || as.moveRandomly) {// if randomize is true, a collision occurred       	
            	 Int2D dir =newDirection(state);
            	 xdir = dir.x;
            	 ydir = dir.y;
            	 randomize = false;
             }
                int newx = x + xdir;// Change the x position by xdir
                int newy = y + ydir;// Change the y position by ydir
                
                newx = hitx(0,as.gridWidth,newx);// Check to see if it hit a wall in
                                           // along the x axis and make the 
                                           // appropriate corrections
                newy = hity(0,as.gridWidth,newy);// Check to see if it hit a wall in
                                           // along the y axis and make the 
                                           // appropriate corrections
                orientation  =lookUpOrientation (xdir,ydir);
                
                /*
                 * Now move the agent
                 */
                	as.agentsSpace.setObjectLocation(this, newx, newy); // set its new  
                	x = newx;
                	y = newy; 
             }                       
        } // end step method
      
     

        
        public int hitx (int boundLow, int boundHigh, int x){
            if (x < boundLow) { 
                x++;  // back up
                xdir = -xdir; // reverse direction
            }
            else if (x >= boundHigh) { 
                x--; // back up
                xdir = -xdir; // reverse direction
            }               
            return x;           
        }
        
        public int hity (int boundLow, int boundHigh, int y){
             if (y < boundLow) { 
                 y++;  // back up
                 ydir = -ydir; // reverse direction
             }
            else if (y >= boundHigh) { 
               y--; // back up
               ydir = -ydir; // reverse direction
               }               
            return y;           
        }         

}