package agents;

 
import java.awt.Color;
import javax.swing.JFrame;

import agents.states.SimStateWithSimController;
import sim.display.Console;
import sim.display.Controller;
import sim.display.Display2D;
import sim.display.GUIState;
import sim.engine.*;
import sim.portrayal.grid.*;
import sim.util.Bag;
 
 
public class GraphicalUI extends GUIState{
        
        public Display2D display; // variable for the graphic display
        public JFrame displayFrame; // for the display frame
        static SimStateWithSimController ssc;
        SparseGridPortrayal2D agentsPortrayal = new SparseGridPortrayal2D();
                // Graphic portrayal of the agents in 2D space             

         protected GraphicalUI() { // Constructor for this class
               super(new AgentsSimulation(System.currentTimeMillis()));
               ssc = (SimStateWithSimController)state;
                         // Create a simulation in it
        }

 
        public GraphicalUI(SimState state) { 
            super(state); // Pass the already created simulation to it
            }
 
        public static String getName() { 
            return "Agents Simulation"; 
                   // return a name for what this simulation is about
             }
 
        public void quit() {
             super.quit(); // Use the already defined quit method
                
             if (displayFrame!=null) displayFrame.dispose(); 
                displayFrame = null;  // when quiting get rid of the display
                display = null;       
        }
 
        public void start() {
              super.start(); // use the predefined start method
               setupPortrayals(); // add setupPortrayals method below
        }

            
        public void load(SimState state) {
            super.load(state); // load the simulation into the interface
            setupPortrayals(); // call setuuPortrayals
        }


                 /*
                 * This is the one method we will change the most for each
.                *  simulation.  Specifically, we determine what agents 
                                   * look like.  
                 */ 
               
        public void setupPortrayals() {
            agentsPortrayal.setField(((AgentsSimulation)state).agentsSpace);
            // Here we give the agentsProtrayal, the agentsSpace so that it
                        // can graphically represent it.

            Bag agents = ((AgentsSimulation)state).agentsSpace.getAllObjects();              //get all the agents
            for(int i=0; i<agents.numObjs;i++){// for all of the agents
               Person a = (Person)agents.get(i); //get each agent from 0 to n

                /*
                 * Now we are going to use RGB colors to set the color
.                * of our agents.  The alpha level determines how  
                                  * transparent the agent is.  
                 */

      
               agentsPortrayal.setPortrayalForObject (a,a.portrayal);        
            } // This complicated statement makes an oval portrayal of
             // a given color
            display.reset(); 
            // call the predefined reset method for the display
            display.repaint(); // call the repaint method
            
            
            ((AgentsSimulation)state).agentsPortrayal = agentsPortrayal;           
            
            
            
         }
 
         public void init(Controller c){
             super.init(c);  // use the predefined method to initialize the
               // controller, c
             display = new Display2D(500,500,this,1); // make the display
               // that is 400 x 400 pixels.  You can set it to other values
             displayFrame = display.createFrame(); // create the frame
             c.registerFrame(displayFrame);   // let the controller control the
              //frame
             displayFrame.setVisible(true);  // make it visible
             display.setBackdrop(Color.white); // make the background black
             display.attach(agentsPortrayal,"Agents Simulation");
                // attach the display
             ssc.setDisplay(display); //pass to display
                }
            
        public Object getSimulationInspectedObject() {
              return state; // This returns the simulation
         }

        
         public static void main(String[] args) {
               GraphicalUI ex = new GraphicalUI(); // make the user interface
                // and call it ex
                Console c = new Console(ex); // make the console and give it
                 // the user interface to display
                ssc.setGUIState(ex); //pass to SimState
                c.setVisible(true); // make the console visible
                ssc.setConsole(c); //Pass to Simstate
                System.out.println("Start Simulation"); // Print that the 
                  // simulation started
                } 

}
