package agents;

 
import java.awt.Color;


import agents.control.SimController;


import sim.engine.Schedule;
import sim.engine.SimState;
import sim.engine.Steppable;
import sim.engine.Stoppable;
import sim.portrayal.simple.OvalPortrayal2D;
import sim.util.Bag;
import sim.util.Int2D;

 
public class Observer implements Steppable {
    public final int LARGE_ORDER = 2147483647; // maximum value of an integer 
    public final double INTERVAL = 1.0;
    public HandleData hd = null;
    public Data data = null;
    public String fileName = "data.txt";
    public String folderName = "data";
    public boolean saveDataToFile = false;
    public Stoppable stop;
    double n = 0.0;
    
    public boolean lastTimeStep = false;
    public HandleData marrAgeHD = null;
    public HandleData matePairsHD = null;
    public HandleData marrAgeIndivHD = null;
    public HandleData marrAgeDatesHD = null;
    public HandleData marrAgeDatesIndivHD = null;  
    public HandleData hdParameters = null; //use a separate file to record parameters.
      
    public SimController sc = null;

    public Observer(SimState state,String fileName, String folderName, 
    				boolean saveDataToFile, SimController sc, String precision){
        stop = state.schedule.scheduleRepeating(this,LARGE_ORDER,INTERVAL);
        this.fileName = fileName;
        this.folderName = folderName;
        this.saveDataToFile= saveDataToFile;
        hd = new HandleData(state, folderName, fileName,saveDataToFile);
        data = new Data();
        
        this.sc = sc;
        
       
        
        String marrAgeFileName = fileName.replace(".txt", "_marrAge.txt");
        marrAgeHD = new HandleData(state, folderName, marrAgeFileName, saveDataToFile);
        
        String matePairFileName = fileName.replace(".txt", "_matePair.txt");
        matePairsHD = new HandleData(state, folderName, matePairFileName, saveDataToFile);
        
        String marrAgeIndivFileName = fileName.replace(".txt", "_marrAgeIndiv.txt");
        marrAgeIndivHD = new HandleData(state, folderName, marrAgeIndivFileName, saveDataToFile);
        
        String marrAgeDatesFileName = fileName.replace(".txt", "_marrAgeDates.txt");
        marrAgeDatesHD = new HandleData(state, folderName, marrAgeDatesFileName, saveDataToFile);
        
        String marrAgeDatesIndivFileName = fileName.replace(".txt", "_marrAgeDatesIndiv.txt");
        marrAgeDatesIndivHD = new HandleData(state, folderName, marrAgeDatesIndivFileName, saveDataToFile);
        
        String par = fileName.replace(".txt", "_parameters.txt");
		hdParameters = new HandleData(state, folderName, par, saveDataToFile);
    }
      
      public Observer(SimState state){
            stop = state.schedule.scheduleRepeating(this,LARGE_ORDER,INTERVAL);
            // make the observe self scheduling
      }
      
      public void doStop (SimState state){
  		AgentsSimulation as = (AgentsSimulation)state;               
  		Bag bag = as.agentsSpace.getAllObjects();
  		for(int i=0;i<bag.numObjs;i++){
  				Person a = (Person)bag.get(i);
  				a.stop.stop();
  		}
  	}
      /***********************************************************************************************
       * 				Begin STEP METHOD
       * We need to synchronize the step for technical reasons since the next step
       * could start before the observer is done.
       */
      public synchronized void step (SimState state){
            AgentsSimulation as = (AgentsSimulation)state;
            final Schedule schedule = state.schedule;
            long thestep = schedule.getSteps();
                   
            Bag people = as.agentsSpace.getAllObjects(); // get all the people
            if(people.isEmpty())
                  stop.stop();
            else{
            	int males = 0;
            	for(int i = 0; i < people.numObjs; i++){
            		Person a = (Person)people.get(i);
            		if(a.sex == Person.MALE){
            			males++;
            			break;
            		}
            	}
            	if(males == 0){ //if there are no males left, stop the sim.
            		doStop(as);
            		stop.stop();	
            	}
            }
            
            //Print data headers to Console
            if(as.writeDataToConsole && thestep == 0){ 
    			if(sc != null && sc.simulationCount == 1){ //the simcount will have been incremented
    				hd.printPramsToConsole(state);
    				hd.printHeadersToConsole(data);
    			}
    			else {
    				hd.printPramsToConsole(state);
    				hd.printHeadersToConsole(data);
    			}
    		}
            
          //Print initial headings to file. 
    		if(saveDataToFile && thestep == 0){ 
    			if(sc != null && sc.simulationCount ==1){ //the simcount will have been incremented
    				//hdParameters
    				String x = "Run: " + sc.runCount;
    				hdParameters.write.writeStringln(x);
    				hdParameters.printPramsToFile(state);
    				if(as.simCount == 1){
    					hd.printHeadersToFile(data);
    				}
    			}	
    		}
            
            
            
            //If at beginning of run, print headers. 
    /*        if(state.schedule.getSteps() == 0){
            	if(as.writeDataToConsole){
            		//print all the simulation parameters.
            		hd.printPramsToConsole(state);                          
            		hd.printHeadersToConsole(data);
            	}
            //	else
            //		System.out.println("Simulation#: " + as.simNumber);
            	
            	if(saveDataToFile && state.schedule.getSteps() == 0 && as.simNumber < 2){
            		hd.printPramsToFile(state);
            		hd.printHeadersToFile(data);
                }
            }*/
            
            
            people = as.agentsSpace.getAllObjects();
            for(int i=0;i <people.numObjs;i++){
            Person p = (Person)people.objs[i];
            if(p.matched && p.match.matched){
                  if(p.sex == Person.MALE)
                        data.getData(p.attractiveness, p.match.attractiveness);
                  else
                        data.getData(p.match.attractiveness, p.attractiveness );
                  
                  //increase the numberMated if maxAttractiveness = 10;
                  if(as.maxAttractiveness > 9 && as.maxAttractiveness < 11){
                      as.numberMated[(int)p.attractiveness-1]++;
                      as.numberMated[(int)p.match.attractiveness-1]++;
                      }
                  //update marriage age.
                  if(p.age < 1000)
                	  as.marriageAge[p.age - 1]++;
                  else 
                	  as.marriageAge[999]++;
                  if(p.match.age < 1000)
                	  as.marriageAge[p.match.age - 1]++;
                  else 
                	  as.marriageAge[999]++;
                //update marriage age in terms of dates
                  if(p.d < 1000)
                	  as.marriageAgeDates[p.d - 1]++;
                  else 
                	  as.marriageAgeDates[999]++;
                  if(p.match.d < 1000)
                	  as.marriageAgeDates[p.match.d - 1]++;
                  else 
                	  as.marriageAgeDates[999]++;
                  //update mate pairs (how well matched everyone is)
                  //I'm not sure if it matters how we order them, but I will always have males
                  //be the rows
                  if(p.sex == Person.MALE)
                	  as.matePairs[(int)p.attractiveness-1][(int)p.match.attractiveness-1]++;
                  else
                	  as.matePairs[(int)p.match.attractiveness-1][(int)p.attractiveness-1]++;
                 
                  as.meanMarriageAgeDates[as.matchCounter] = p.d;
                  as.meanMarriageAgeDates[as.matchCounter + 1] = p.match.d;
                  as.meanMarriageAgeTime[as.matchCounter] = p.age;
                  as.meanMarriageAgeTime[as.matchCounter + 1] = p.match.age;
                  as.matchCounter += 2;
                  
                  
                  //update marriageAgeIndiv and marriageAgeDatesIndiv
                  final int attr1 = (int)p.attractiveness;
                  final int attr2 = (int)p.match.attractiveness;
                  as.marriageAgeIndiv[attr1-1][as.marriageAgeIndivCounter[attr1-1]] = p.age;
                  as.marriageAgeDatesIndiv[attr1-1][as.marriageAgeIndivCounter[attr1-1]] = p.d;
                  as.marriageAgeIndivCounter[attr1-1]++;
                  as.marriageAgeIndiv[attr2-1][as.marriageAgeIndivCounter[attr2-1]] = p.match.age;
                  as.marriageAgeDatesIndiv[attr2-1][as.marriageAgeIndivCounter[attr2-1]] = p.match.d;
                  as.marriageAgeIndivCounter[attr2-1]++;
                  
                  
                p.match.remove(state);
                p.remove(state); //remove both people
                p.match.matched = false;
                p.matched = false;
                if(as.replacement)
                	replaceAgents(as); //replace the lost agents with new agents. 
                  }
            }
            data.correlation();
            data.means();
            data.t_value();
            data.getData(state);
            
            lastTimeStep = isLastTimeStep(as);//have we gotten to the end of the run?
            if(as.autoSimulation){
				data.runCount = sc.runCount;
				data.simCount = sc.simulationCount;
			}
			else{
				data.runCount = 0;
				data.simCount = as.simCount;	
			}
            
            
            
            
         // has something new been added since last time?
            if((n !=  ((Double)data.data.objs[3]).doubleValue() || !as.prettierAtClosingTime) && !as.printedInfo){ 
            	if(as.writeDataToConsole)
            		hd.printDataToConsole(data);
            	//when printOnlyEndResult, only print if ALL the agents have been counted.
                 if(as.saveDataToFile && (!as.printOnlyEndResult || lastTimeStep ))
                        hd.printDataToFile(data);
                 if(as.saveDataToFile && lastTimeStep){
                	 printOtherStats(as);
                	 as.printedInfo = true;
                 }
                  n = ((Double)data.data.objs[1]).doubleValue();  
                  
            }
            
            people = as.agentsSpace.getAllObjects();
            for(int j=0; j<people.numObjs;j++){
                  Person p = (Person)people.objs[j];
                  p.attemptedMatch = false;
               //   p.age++; //increment age (different than # of dates for spatial condition).
                  
            } //end for
            
           /* if(state.schedule.getSteps()%100==0){
            	System.out.println("1\t2\t3\t4\t5\t6\t7\t8\t9\t10");
            	for(int i=0; i < 10; i++){
            		int m = as.numberMated[i];
            		System.out.print(m + "\t");
            	}
            	System.out.println("");
            }*/
            
      }
      /*.**********************************************************************************************
       * 							End STEP METHOD
       ************************************************************************************/
 
      
      public void printOtherStats(AgentsSimulation as){
    	//print other stats to file.
          	//print marriageAge          
          	if(as.simNumber < 2){
          		//print headers
          		String header = "run" + "\t" + "sim" + "\t";
          		for(int i = 0; i < as.marriageAge.length; i++)
          			header += (i+1) + "\t";
          		marrAgeHD.write.writeStringln(header);
          		marrAgeDatesHD.write.writeStringln(header);
          	}
          	String theData = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	String theData1 = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	for(int i = 0; i < as.marriageAge.length; i++){
          		theData += as.marriageAge[i] + "\t";
          		theData1 += as.marriageAgeDates[i] + "\t";
          	}
          	marrAgeHD.write.writeStringln(theData);	
          	marrAgeDatesHD.write.writeStringln(theData1);	
          	
          	boolean doMatePairs = false; //this takes up a lot of memory.
          	
          	//print matePairs
          	//matePairsHD.write.writeStringln("Simulation" + "\t" + as.simNumber);
          	final int maxAttr = (int)as.maxAttractiveness;
          	String header2 = "run " + (sc.runCount+1) + ", sim " + sc.simulationCount + "\t";
          	if(doMatePairs){
          	for(int i = 0; i < maxAttr; i++)
          		header2 += (i+1) + "\t";
          	matePairsHD.write.writeStringln(header2);//write the header. The corner is the simNumber
          	
          	String thisData = "";
          	for(int i = 0; i < maxAttr; i++){ //for every row
          		thisData = (i+1) + "\t";
          		for(int j = 0; j < maxAttr; j++)//go through each column
          			thisData += as.matePairs[i][j] + "\t";
          		matePairsHD.write.writeStringln(thisData);
          	}
          	}
          	//deal with marriageAgeIndiv.
          	//calculate the mean and SD for each category. 
          	if(sc.simulationCount < 2){
          	String header3 = "run" + "\t" + "sim" + "\t";
          	for(int i = 0; i < maxAttr; i++)
          		header3 += (i+1) + "\t";
          	marrAgeIndivHD.write.writeStringln(header3);
          	marrAgeDatesIndivHD.write.writeStringln(header3);
          	}
          	double[][] tempStats = new double [maxAttr][2]; //will hold the mean and SD of marriage age.
          	for(int j = 0; j < maxAttr; j++){
          		final int [] marrTemp = as.marriageAgeIndiv[j];
          		final int size = as.marriageAgeIndivCounter[j];
          		final double avg = mean(marrTemp, size);
          		tempStats[j][0] = avg;
          		tempStats[j][1] = stdev(marrTemp, size, avg);
          	}
          	String meanData = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	String sdData = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	for(int i = 0; i < maxAttr; i++){
          		meanData += tempStats[i][0] + "\t";
          		sdData += tempStats[i][1] + "\t";
          	}
          	marrAgeIndivHD.write.writeStringln(meanData);
          	//marrAgeIndivHD.write.writeStringln(sdData);
          	
          	double[][] tempStats2 = new double [maxAttr][2]; //will hold the mean and SD of marriage age.
          	for(int j = 0; j < maxAttr; j++){
          		final int [] marrTemp = as.marriageAgeDatesIndiv[j];
          		final int size = as.marriageAgeIndivCounter[j];
          		final double avg = mean(marrTemp, size);
          		tempStats2[j][0] = avg;
          		tempStats2[j][1] = stdev(marrTemp, size, avg);
          	}
          	String meanData2 = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	String sdData2 = (sc.runCount+1) + "\t" + sc.simulationCount + "\t";
          	for(int i = 0; i < maxAttr; i++){
          		meanData2 += tempStats2[i][0] + "\t";
          		sdData2 += tempStats2[i][1] + "\t";
          	}
          	marrAgeDatesIndivHD.write.writeStringln(meanData2);
          	//marrAgeDatesIndivHD.write.writeStringln(sdData2);
      }
      
      public boolean isLastTimeStep(AgentsSimulation as){
    	  if((!as.replacement && data.males > (as.males -1)) || (as.replacement && as.schedule.getSteps() > 999))
    			 // || (!as.prettierAtClosingTime && as.schedule.getSteps() > 999998)) //we've matched all the agents
    		  return true;
    	  if(as.schedule.getSteps() == 999999){
    		  System.out.println("Hooray!");
    		  return true;
    	  }
    		  
    	  //allow for future possiblities, such as a time limit.
    	  else
    		  return false;
      }
      
      /**
       * 
       */
      public void replaceAgents(SimState state){
    	  AgentsSimulation as = (AgentsSimulation)state;
    	  //Male
    	  int x = as.random.nextInt(as.gridWidth); // a random number < gridWidth
    	  int y = as.random.nextInt(as.gridHeight); //a random number < gridHeight
    	  Int2D dir = as.randomMovement[as.random.nextInt(8)];
    	  Person m = new Person(x,y,dir.x,dir.y);
    	  m.sex = Person.MALE;
    	  m.portrayal = new OvalPortrayal2D(new Color(0, 0, 0, 255), 1);   // define a color 
    	  m.d = as.initD();
    	  if(!as.normalDist)
    		  m.attractiveness = (double)as.random.nextInt((int)as.maxAttractiveness)+1;
    	  else
    		  m.attractiveness = as.normalAttr();
    	  m.movementProbs = as.movementProbsSums;
    	  m.orientationLookUp = as.orientationLookUp;
    	  m.directionLookUp = as.directionLookUp;
    	  m.randomMovement = as.randomMovement;         	   
    	  m.orientation = m.lookUpOrientation(dir.x,dir.y);
    	  
    	  //In order to have replacement agents be the same color as initial agents, 
    	  //I need to set up a portrayal in the State and have it correspond to the one in the UI. 
    	  //Just follow the way it's done in game_agents_evolve
    	  
    	  as.agentsPortrayal.setPortrayalForObject(m, new OvalPortrayal2D(new Color(0, 0, 255, 255), 1));
    	  as.agentsSpace.setObjectLocation(m, x,y); //put it at location x, y
    	  m.stop = as.schedule.scheduleRepeating(m); // schedule it to repeat in the simulation
    	//Female
    	  x = as.random.nextInt(as.gridWidth); // a random number < gridWidth
    	  y = as.random.nextInt(as.gridHeight); //a random number < gridHeight
    	  dir = as.randomMovement[as.random.nextInt(8)];
    	  Person f = new Person(x,y,dir.x,dir.y);
    	  f.sex = Person.FEMALE;
    	  //f.portrayal = new OvalPortrayal2D(new Color(0, 0, 1, 1), 1);   // define a color 
    	  f.d = as.initD();
    	  if(!as.normalDist)
    		  f.attractiveness = (double)as.random.nextInt((int)as.maxAttractiveness)+1;
    	  else
    		  f.attractiveness = as.normalAttr();
    	  f.movementProbs = as.movementProbsSums;
    	  f.orientationLookUp = as.orientationLookUp;
    	  f.directionLookUp = as.directionLookUp;
    	  f.randomMovement = as.randomMovement;         	   
    	  f.orientation = f.lookUpOrientation(dir.x,dir.y);
    	  as.agentsPortrayal.setPortrayalForObject(f, new OvalPortrayal2D(new Color((float)1.0, (float)0.08 ,(float)0.57, 1), 1));
    	  
    	  as.agentsSpace.setObjectLocation(f, x,y); //put it at location x, y
    	  f.stop = as.schedule.scheduleRepeating(f); // schedule it to repeat in the simulation
      }
      
      /**
       * calculates the mean of an array of integers.
       * we need to specify the size here because some of the later entries might be blank (zero) 
       */
      public double mean(int[] vector, int size){
    	 double val = 0.0;
    	 for(int i = 0; i < size; i++)
    		  val += (double)vector[i];
    	 val = val/(double)size;
    	 return val;
      }
      
      /**
       * calculates the standard deviation of an array of integers.
       * we need to specify the size here because some of the later entries might be blank (zero) 
       */
      public double stdev(int[] vector, int size, double avg){
    	  double val = 0.0;
    	  double sumSq = 0.0;
    	  for(int i = 0; i < size; i++)
    		  sumSq += Math.pow((double)(vector[i] - avg), 2.0); //sum of the squares.
    	  val = (1/(double)(size - 1))*sumSq;
    	  val = Math.sqrt(val);
    	  return val;
      }

}