package agents;

import java.awt.Color;

import sim.engine.SimState;
import sim.engine.Stoppable;
import sim.portrayal.SimplePortrayal2D;
import sim.portrayal.simple.OvalPortrayal2D;
import sim.util.Bag;
import java.util.Random;

public class Person extends Agent{
	
	 public final static int MALE = 0;
     public final static int FEMALE = 1;
     public int sex;
     public SimplePortrayal2D portrayal = null;
     public Stoppable stop;
     public double attractiveness;
     public int d = 1; // # of dates
     public boolean attemptedMatch = false;
     public boolean matched = false;
     public Person match = null;
     public int age = 1; //this will be different than d for spatial condition


     public int getsex (){   return sex; }
     public double getAttractiveness(){ return attractiveness; }
     public int getd(){	return d;}
     //public SimplePortrayal2D getPortrayal(){ return portrayal;}
     
	public Person(int x, int y, int xdir, int ydir) {
		super(x, y, xdir, ydir);
	}
	
	public synchronized void step(SimState state){
		super.step(state); //move
		
		if(!this.attemptedMatch){ //if I haven't dated this time step...
			AgentsSimulation as = (AgentsSimulation)state;
			Bag allAgents= new Bag();
			
			allAgents = as.agentsSpace.getAllObjects();
			Bag neighbors = new Bag();
			if(as.maxMatchRadius || allAgents.numObjs == 2)
				neighbors = as.agentsSpace.getAllObjects();
			else
				neighbors = as.agentsSpace.getNeighborsMaxDistance(x,y, as.matchRadius,false, neighbors, null, null);
			Bag oppositeSex = findOppositeSex(neighbors);
			Person date = null;
			if(oppositeSex.numObjs == 1)
				date = (Person)oppositeSex.objs[0]; //get that person
			else if(oppositeSex.numObjs > 1)
				date = (Person)oppositeSex.objs[state.random.nextInt(oppositeSex.numObjs)]; //get that person
			if(date != null){
				matchAttractive(state, date); // try match
			}
		}
		if(!this.matched)
			this.age++;
	}//end step method


	
	
	
	
	public Bag findOppositeSex(Bag neighbors){
        Bag oppositeSex = new Bag();
        for(int i=0;i< neighbors.numObjs;i++){
              Person p = (Person)neighbors.objs[i];
              if(p.sex != this.sex && !p.attemptedMatch)    
                   oppositeSex.add(p);
             }
             return oppositeSex;
       }
	
	
	public void matchAttractive(SimState state, Person date){
    AgentsSimulation as = (AgentsSimulation)state;  
    
    if(chooseAttractive(state,date) && date.chooseAttractive(state, this)){
          match = date;
          date.match = this; // make sure each person knows its match
          matched = true;
          date.matched = true;
          
    }
    else{
    	d++;
    	date.d++;
    }
    attemptedMatch = true; // only want one attempt on each round
    date.attemptedMatch = true;
    
}
	
	/**
	 * Make a portrayal for the new agent. This is used by the start method in agentsSimulation.
	 */
	public void makePortrayl(float red, float green, float blue, float alpha){
         this.portrayal = new OvalPortrayal2D (new 
                    Color(red,green,blue,alpha),2);
   } // make a portrayal

	
	/**
	 * Remove an agent from both space and from the list. Used by the Observer agent.
	 */
	public void remove(SimState state){
         AgentsSimulation as = (AgentsSimulation)state;
         as.agentsSpace.remove(this);  // remove it from space
             if(stop != null)
                 stop.stop();   // remove it from the list
      } 
	 
	 /***********************************************************************************
	  * 		MATCHING MEASURES
	  ***********************************************************************************/
	 public double p1(SimState state, Person date){ //Most attractive
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(this.d < 1 || date.d < 1)
			 return 0; //let some people be "not ready"
		 else if (this.d >= as.max_d)
			 return 1;
		 else{
			 
			 return Math.pow(date.attractiveness, as.exponent)/Math.pow(as.maxAttractiveness, as.exponent);
		 }
	 }
	 
	 public double p1c(SimState state, Person date){
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(this.d < 1 || date.d < 1)
			 return 0; //let some people be "not ready"
		 else if(this.d >= as.max_d)
			 return 1;
		 else{
		
		 if(as.prettierAtClosingTime)
			 return Math.pow(p1(state,date), ((double)as.max_d - (double)d)/(double)as.divisor);
		 else
			 return p1(state,date);
		 }
	 }
	 
	 public double p2(SimState state, Person date){ //The matching hypothesis: similarity
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(this.d < 1 || date.d < 1)
			 return 0; //let some people be "not ready"
		 else if(this.d >= as.max_d)
			 return 1;
		 else{
			 return Math.pow(as.maxAttractiveness-Math.abs(this.attractiveness-date.attractiveness), as.exponent)
		 		/Math.pow(as.maxAttractiveness, as.exponent);		 
		 }
	 }
	 
	 public double p2c(SimState state, Person date){
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(this.d < 1 || date.d < 1)
			 return 0; //let some people be "not ready"
		 else if(this.d >= as.max_d)
			 return 1;
		 else{
		 if(as.prettierAtClosingTime)
			 return Math.pow(p2(state,date), ((double)as.max_d - (double)d)/(double)as.divisor);
		 else
			 return p2(state,date);
		 }
	}
	 
	 public double p3(SimState state, Person date){ //Average of 1st 2 methods
		 AgentsSimulation as = (AgentsSimulation)state;
		 return (p1(state,date)+p2(state,date))/2.0;
		 
	 }
	 
	 public double p3c(SimState state, Person date){
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(as.prettierAtClosingTime)
			 return Math.pow(p3(state,date), ((double)as.max_d - (double)d)/(double)as.divisor);
		 else
			 return p3(state,date);
	 }
	 
	 public double p4(SimState state, Person date){ // probabilistically uses one of 1st 2 methods.
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(state.random.nextDouble() <= as.p)
			 return p1(state,date);
		 else
			 return p2(state, date);
	 }
	 
	 public double p4c(SimState state, Person date){
		 AgentsSimulation as = (AgentsSimulation)state;
		 if(as.prettierAtClosingTime)
			 return Math.pow(p4(state,date), ((double)as.max_d - (double)d)/(double)as.divisor);
		 else
			 return p4(state,date);
	 }
	 /*.**********************************************************************************
	  * 		End MATCHING MEASURES
	  ***********************************************************************************/
	 
	 
	 /**
	  * Does the agent select its date for a match?
	  */
	 public boolean chooseAttractive(SimState state, Person date){
		 AgentsSimulation as = (AgentsSimulation)state;

		 double p = state.random.nextDouble(); //Fast Mersenne Twister (default)
		 
		if(as.rule == AgentsSimulation.ATTRACTIVE){
			 if(p <= p1c(state,date))
				 return true; //accepts
			 else
				 return false; //declines
		 }
		 else if(as.rule == AgentsSimulation.SIMILAR){
			 if(state.random.nextBoolean(p2c(state,date)))
				 return true; //accepts
			 else
				 return false; //declines
		 }
		 else if(as.rule == AgentsSimulation.MIXED){
			 if(state.random.nextBoolean(p3c(state,date)))
				 return true; //accepts
			 else
				 return false; //declines
		 }
		 else if(as.rule == AgentsSimulation.MIXED2){
			 if(state.random.nextBoolean(p4c(state,date)))
				 return true; //accepts
			 else
				 return false; //declines
		 }
		 else{ //default to attractiveness
			 System.err.println("No Rule: " + as.rule);
			 if(state.random.nextBoolean(p1c(state,date)))
				 return true; //accepts
			 else
				 return false; //declines
		 }
	 }
	 
	 

}// end Class
